from AccessControl import ClassSecurityInfo
from Acquisition import aq_base, aq_parent, aq_inner
from BTrees.OOBTree import OOBTree
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
from ZODB.PersistentMapping import PersistentMapping
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import UniqueObject, getToolByName

from DateTime import DateTime

from Products.Archetypes.debug import log

from config import *

StringType = type('')

CATALOG_TOOL   = "portal_catalog"
MEMBERSHIP_TOOL = "portal_membership"
REFERENCE_TOOL = "reference_catalog"
ARCHETYPE_TOOL = "archetype_tool"

STICKY_META_TYPE = "Sticky"


class StickiesTool(UniqueObject, SimpleItem):
    security=ClassSecurityInfo()

    id = TOOL_NAME

    title = TOOL_TITLE
    meta_type = TOOL_TITLE

    manage_options = SimpleItem.manage_options

    def __init__(self):
        pass
    
    def _genId(self, hidden):
        """
        stick with the plone mechanism, for now
        """
        # too bad we cant call plone_scripts/generateId directly
        now = DateTime()
        prefix = ""
        # plone folders hide stuff w/ '.' prefix
        if hidden:
            prefix = "."
            
        id = prefix + "%s.%s.%s" % ("sticky", now.strftime('%Y-%m-%d'), now.strftime('%M%S'))
        return id 

    def _getMemberId(self):
        membership = getToolByName(self, MEMBERSHIP_TOOL)
        try:
            memberId = membership.getAuthenticatedMember().getMemberId()
        except:
            memberId = None

        return memberId

    security.declarePublic("createNewSticky")
    def createNewSticky(self, target, redirect=False, hidden=True, REQUEST=None):
        """
        create a new sticky object, stored in the user's home directory
        target -  either the uid, or the actual object itself, of the thing being stuck to
        """
        membership = getToolByName(self, MEMBERSHIP_TOOL)
        ref_catalog = getToolByName(self, REFERENCE_TOOL)
        memberId = self._getMemberId()

        home = membership.getHomeFolder(id=memberId)

        sticky_id = self._genId(hidden)

        # store the sticky note in the users home dir
        home.invokeFactory(type_name=STICKY_META_TYPE, id=sticky_id)
        sticky = getattr(home, sticky_id, None)
        if not sticky:
            return None

        # if they pass in a uid, look up the object
        if type(target) == StringType:
            targetObject = ref_catalog.lookupObject(target)
        else:
            targetObject = target

        # connect the sticky note to the target content        
        sticky.addReference(targetObject, STICKY_RELATIONSHIP)
        sticky.setTitle("Sticky on %s" % targetObject.Title())

        if REQUEST and redirect:
            return REQUEST.RESPONSE.redirect("%s" % sticky.absolute_url())

        return sticky
        
    security.declarePublic("deleteSticky")
    def deleteSticky(self, stickyUID, redirect=0, REQUEST=None):
        """delete a Stickey note with the given uid"""
        portal = self.portal_url.getPortalObject()
        
        ref_catalog = getToolByName(self, REFERENCE_TOOL)
        sticky = ref_catalog.lookupObject(stickyUID)

        container = aq_parent(aq_inner(sticky))
        try:
            container._delObject(sticky.id)
        except:
            return False
        
        if REQUEST and redirect:
            # maybe back to the portal home? not sure
            return REQUEST.RESPONSE.redirect("%s" % portal.absolute_url())
        
        return True
    
    security.declarePublic("findStickiesByTarget")
    def findStickiesByTarget(self, targetUID, review_states=None):
        """returns dictionary of sticky objects, keyed off of member ids"""
        stickies = {}

        ref_catalog = getToolByName(self, REFERENCE_TOOL)
        # find all sticky relationships on this target
        sticky_refs = ref_catalog(relationship = STICKY_RELATIONSHIP, targetUID = targetUID)

        # lookup the sticky notes themselves
        results = []
        for ref in sticky_refs:
            o = ref_catalog.lookupObject(ref.sourceUID)
            # review_state should realy be looked up in the query,
            # but that index isn't on the ref catalog.
            # maybe put it on there later
            if review_states:
                if o.getReviewState() in review_states:
                    results.append(o)
            else:
                results.append(o)
        
        # return them in a dict, keyed off of member ids
        stickies = dict([(i.Creator(), i) for i in results])
        return stickies

    security.declarePublic("findStickiesByMember")
    def findStickiesByMember(self, memberId):
        """returns dictionary of stickies ojbects, keyed off of target ids"""
        stickies = {}
        catalog = getToolByName(self, CATALOG_TOOL)

        # lookup all Stickys in the regular catalog created by memberId
        sticky_brains = catalog(Type = STICKY_META_TYPE, Creator = memberId)
        results = [i.getObject() for i in sticky_brains]

        # now lookup the target object this not is stuck to
        for r in results:
            target = r.getRefs(relationship = STICKY_RELATIONSHIP)[0]
            stickies[target.UID()] = r
        
        return stickies

    security.declarePublic("getSticky")
    def getSticky(self, target, memberId=None):
        """
        returns the sticky attached to an object, for a particular member
        target - either the uid or the object itself
        """

        sticky = None

        catalog = getToolByName(self, CATALOG_TOOL)
        ref_catalog = getToolByName(self, REFERENCE_TOOL)

        # if no memberId is passed, use 'me'
        if not memberId:
            memberId = self._getMemberId()

        #log("target=%s, memberId=%s" % (target, memberId))

        # if they pass in an object, get its uid
        if type(target) == StringType:
            targetUID = target
        else:
            targetUID = target.UID()

        # find all sticky relationships on this target
        sticky_refs = ref_catalog(relationship = STICKY_RELATIONSHIP, targetUID = targetUID)
        sticky_uids = [i.sourceUID for i in sticky_refs]

        # log ("sticky_uids = %s" % sticky_uids)
        # stickies installer creates a UID index on the main catalog
        results = catalog(Creator = memberId,
                          UID = { "query" : sticky_uids } )

        # log ("results = %s" % results)
        if results:
            # for now, we only are creating one note, per object, per user
            # in later versions we will allow for positional placement of notes
            sticky = results[0].getObject()

        #log ("sticky = %s" % sticky)
        return sticky
                          
InitializeClass(StickiesTool)
